@@ -4,8 +4,8 @@ use crate::history::History;
 use crate::fixed_length_grapheme_string::FixedLengthGraphemeString;
 use crate::history::Command;
 use crate::history_cleaner;
-use crate::settings::KeyScheme;
 use crate::settings::Settings;
+use crate::settings::{InterfaceView, KeyScheme};
 use chrono::{Duration, TimeZone, Utc};
 use humantime::format_duration;
 use std::io::{stdin, stdout, Write};
@@ -56,9 +56,9 @@ impl MenuMode {
                 KeyScheme::Emacs => "McFly | ESC - Exit | ⏎ - Run | TAB - Edit | F2 - Delete",
                 KeyScheme::Vim => {
                     if interface.in_vim_insert_mode {
-                        "McFly (Vim) | ESC - Exit | ⏎ - Run | TAB - Edit | F2 - Delete        -- INSERT --"
+                        "McFly (Ins) | ESC - Cmd  | ⏎ - Run | TAB - Edit | F2 - Delete"
                     } else {
-                        "McFly (Vim) | ESC - Exit | ⏎ - Run | TAB - Edit | F2 - Delete"
+                        "McFly (Cmd) | ESC - Exit | ⏎ - Run | TAB - Edit | F2 - Delete"
                     }
                 }
             },
@@ -133,24 +133,27 @@ impl<'a> Interface<'a> {
     }
 
     fn menubar<W: Write>(&self, screen: &mut W) {
-        let (width, _height): (u16, u16) = terminal_size().unwrap();
-        write!(
-            screen,
-            "{hide}{cursor}{clear}{fg}{bg}{text:width$}{reset_bg}",
-            hide = cursor::Hide,
-            fg = color::Fg(color::LightWhite).to_string(),
-            bg = self.menu_mode.bg(),
-            cursor = cursor::Goto(1, INFO_LINE_INDEX),
-            clear = clear::CurrentLine,
-            text = self.menu_mode.text(self),
-            reset_bg = color::Bg(color::Reset).to_string(),
-            width = width as usize
-        )
-        .unwrap();
-        screen.flush().unwrap();
+        if !self.settings.disable_menu {
+            let (width, _height): (u16, u16) = terminal_size().unwrap();
+            write!(
+                screen,
+                "{hide}{cursor}{clear}{fg}{bg}{text:width$}{reset_bg}",
+                hide = cursor::Hide,
+                fg = color::Fg(color::LightWhite),
+                bg = self.menu_mode.bg(),
+                cursor = cursor::Goto(1, self.info_line_index()),
+                clear = clear::CurrentLine,
+                text = self.menu_mode.text(self),
+                reset_bg = color::Bg(color::Reset),
+                width = width as usize
+            )
+            .unwrap();
+            screen.flush().unwrap();
+        }
     }
 
     fn prompt<W: Write>(&self, screen: &mut W) {
+        let prompt_line_index = self.prompt_line_index();
         write!(
             screen,
             "{}{}{}$ {}",
@@ -159,7 +162,7 @@ impl<'a> Interface<'a> {
             } else {
                 color::Fg(color::LightWhite).to_string()
             },
-            cursor::Goto(1, PROMPT_LINE_INDEX),
+            cursor::Goto(1, self.prompt_line_index()),
             clear::CurrentLine,
             self.input
         )
@@ -167,7 +170,7 @@ impl<'a> Interface<'a> {
         write!(
             screen,
             "{}{}",
-            cursor::Goto(self.input.cursor as u16 + 3, PROMPT_LINE_INDEX),
+            cursor::Goto(self.input.cursor as u16 + 3, prompt_line_index),
             cursor::Show
         )
         .unwrap();
@@ -175,22 +178,24 @@ impl<'a> Interface<'a> {
     }
 
     fn debug_cursor<W: Write>(&self, screen: &mut W) {
+        let result_top_index = self.result_top_index();
         write!(
             screen,
             "{}{}",
             cursor::Hide,
-            cursor::Goto(0, RESULTS_TOP_INDEX + self.settings.results + 1)
+            cursor::Goto(0, result_top_index + self.settings.results + 1)
         )
         .unwrap();
         screen.flush().unwrap();
     }
 
     fn results<W: Write>(&mut self, screen: &mut W) {
+        let result_top_index = self.result_top_index();
         write!(
             screen,
             "{}{}{}",
             cursor::Hide,
-            cursor::Goto(1, RESULTS_TOP_INDEX),
+            cursor::Goto(1, result_top_index),
             clear::All
         )
         .unwrap();
@@ -229,10 +234,15 @@ impl<'a> Interface<'a> {
 
             write!(screen, "{}{}", fg, bg).unwrap();
 
+            let command_line_index = self.command_line_index(index as i16);
+
             write!(
                 screen,
                 "{}{}",
-                cursor::Goto(1, index as u16 + RESULTS_TOP_INDEX),
+                cursor::Goto(
+                    1,
+                    (command_line_index as i16 + result_top_index as i16) as u16
+                ),
                 Interface::truncate_for_display(
                     command,
                     &self.input.command,
@@ -248,7 +258,10 @@ impl<'a> Interface<'a> {
                 write!(
                     screen,
                     "{}",
-                    cursor::Goto(width - 9, index as u16 + RESULTS_TOP_INDEX)
+                    cursor::Goto(
+                        width - 9,
+                        (command_line_index as i16 + result_top_index as i16) as u16
+                    )
                 )
                 .unwrap();
 
@@ -310,14 +323,27 @@ impl<'a> Interface<'a> {
     }
 
     fn move_selection(&mut self, direction: MoveSelection) {
-        match direction {
-            MoveSelection::Up => {
-                if self.selection > 0 {
-                    self.selection -= 1;
+        if self.is_screen_view_bottom() {
+            match direction {
+                MoveSelection::Up => {
+                    self.selection += 1;
+                }
+                MoveSelection::Down => {
+                    if self.selection > 0 {
+                        self.selection -= 1;
+                    }
                 }
             }
-            MoveSelection::Down => {
-                self.selection += 1;
+        } else {
+            match direction {
+                MoveSelection::Up => {
+                    if self.selection > 0 {
+                        self.selection -= 1;
+                    }
+                }
+                MoveSelection::Down => {
+                    self.selection += 1;
+                }
             }
         }
     }
@@ -355,6 +381,7 @@ impl<'a> Interface<'a> {
             &self.input.command,
             self.settings.results as i16,
             self.settings.fuzzy,
+            &self.settings.result_sort,
         );
     }
 
@@ -472,7 +499,11 @@ impl<'a> Interface<'a> {
             }
             Key::F(2) => {
                 if !self.matches.is_empty() {
-                    self.menu_mode = MenuMode::ConfirmDelete;
+                    if self.settings.delete_without_confirm {
+                        self.delete_selection();
+                    } else {
+                        self.menu_mode = MenuMode::ConfirmDelete;
+                    }
                 }
             }
             _ => {}
@@ -501,8 +532,10 @@ impl<'a> Interface<'a> {
                 }
                 Key::Left => self.input.move_cursor(Move::Backward),
                 Key::Right => self.input.move_cursor(Move::Forward),
-                Key::Up | Key::PageUp | Key::Ctrl('u') => self.move_selection(MoveSelection::Up),
-                Key::Down | Key::PageDown | Key::Ctrl('d') => {
+                Key::Up | Key::PageUp | Key::Ctrl('u') | Key::Ctrl('p') => {
+                    self.move_selection(MoveSelection::Up)
+                }
+                Key::Down | Key::PageDown | Key::Ctrl('d') | Key::Ctrl('n') => {
                     self.move_selection(MoveSelection::Down)
                 }
                 Key::Esc => self.in_vim_insert_mode = false,
@@ -522,7 +555,11 @@ impl<'a> Interface<'a> {
                 }
                 Key::F(2) => {
                     if !self.matches.is_empty() {
-                        self.menu_mode = MenuMode::ConfirmDelete;
+                        if self.settings.delete_without_confirm {
+                            self.delete_selection();
+                        } else {
+                            self.menu_mode = MenuMode::ConfirmDelete;
+                        }
                     }
                 }
                 _ => {}
@@ -574,7 +611,11 @@ impl<'a> Interface<'a> {
                 }
                 Key::F(2) => {
                     if !self.matches.is_empty() {
-                        self.menu_mode = MenuMode::ConfirmDelete;
+                        if self.settings.delete_without_confirm {
+                            self.delete_selection();
+                        }else{
+                            self.menu_mode = MenuMode::ConfirmDelete;
+                        }
                     }
                 }
                 _ => {}
@@ -653,6 +694,42 @@ impl<'a> Interface<'a> {
 
         out.string
     }
+
+    fn result_top_index(&self) -> u16 {
+        let (_width, height): (u16, u16) = terminal_size().unwrap();
+
+        if self.is_screen_view_bottom() {
+            return height - RESULTS_TOP_INDEX;
+        }
+        RESULTS_TOP_INDEX
+    }
+
+    fn prompt_line_index(&self) -> u16 {
+        let (_width, height): (u16, u16) = terminal_size().unwrap();
+        if self.is_screen_view_bottom() {
+            return height - PROMPT_LINE_INDEX;
+        }
+        PROMPT_LINE_INDEX
+    }
+
+    fn info_line_index(&self) -> u16 {
+        let (_width, height): (u16, u16) = terminal_size().unwrap();
+        if self.is_screen_view_bottom() {
+            return height;
+        }
+        INFO_LINE_INDEX
+    }
+
+    fn command_line_index(&self, index: i16) -> i16 {
+        if self.is_screen_view_bottom() {
+            return -index;
+        }
+        index
+    }
+
+    fn is_screen_view_bottom(&self) -> bool {
+        self.settings.interface_view == InterfaceView::Bottom
+    }
 }
 
 // TODO:
