@@ -2,7 +2,7 @@ use crate::shell_history;
 use clap::AppSettings;
 use clap::{crate_authors, crate_version, value_t};
 use clap::{App, Arg, SubCommand};
-use dirs::home_dir;
+use directories_next::{ProjectDirs, UserDirs};
 use std::env;
 use std::path::PathBuf;
 use std::str::FromStr;
@@ -31,6 +31,18 @@ pub enum InitMode {
     Fish,
 }
 
+#[derive(Debug, PartialEq)]
+pub enum InterfaceView {
+    Top,
+    Bottom,
+}
+
+#[derive(Debug)]
+pub enum ResultSort {
+    Rank,
+    LastRun,
+}
+
 #[derive(Debug, Clone, Copy)]
 pub enum HistoryFormat {
     /// bash format - commands in plain text, one per line, with multi-line commands joined.
@@ -53,7 +65,7 @@ pub enum HistoryFormat {
 pub struct Settings {
     pub mode: Mode,
     pub debug: bool,
-    pub fuzzy: bool,
+    pub fuzzy: i16,
     pub session_id: String,
     pub mcfly_history: PathBuf,
     pub output_selection: Option<String>,
@@ -71,6 +83,10 @@ pub struct Settings {
     pub limit: Option<i64>,
     pub skip_environment_check: bool,
     pub init_mode: InitMode,
+    pub delete_without_confirm: bool,
+    pub interface_view: InterfaceView,
+    pub result_sort: ResultSort,
+    pub disable_menu: bool,
 }
 
 impl Default for Settings {
@@ -89,13 +105,17 @@ impl Default for Settings {
             refresh_training_cache: false,
             append_to_histfile: false,
             debug: false,
-            fuzzy: false,
+            fuzzy: 0,
             lightmode: false,
             key_scheme: KeyScheme::Emacs,
             history_format: HistoryFormat::Bash,
             limit: None,
             skip_environment_check: false,
             init_mode: InitMode::Bash,
+            delete_without_confirm: false,
+            interface_view: InterfaceView::Top,
+            result_sort: ResultSort::Rank,
+            disable_menu: false,
         }
     }
 }
@@ -123,7 +143,7 @@ impl Settings {
                 .takes_value(true))
             .arg(Arg::with_name("history_format")
                 .long("history_format")
-                .help("Shell history file format, 'bash', 'zsh', or 'fish' (defaults to 'bash')")
+                .help("Shell history file format, 'bash', 'zsh', 'zsh-extended' or 'fish' (defaults to 'bash')")
                 .value_name("FORMAT")
                 .takes_value(true))
             .subcommand(SubCommand::with_name("add")
@@ -137,10 +157,7 @@ impl Settings {
                     .takes_value(true))
                 .arg(Arg::with_name("append_to_histfile")
                     .long("append-to-histfile")
-                    .help("Also append new history to $HISTFILE (e.q., .bash_history)"))
-                .arg(Arg::with_name("zsh_extended_history")
-                    .long("zsh-extended-history")
-                    .help("If appending, use zsh's EXTENDED_HISTORY format"))
+                    .help("Also append new history to $HISTFILE/$MCFLY_HISTFILE (e.q., .bash_history)"))
                 .arg(Arg::with_name("when")
                     .short("w")
                     .long("when")
@@ -183,7 +200,10 @@ impl Settings {
                 .arg(Arg::with_name("fuzzy")
                     .short("f")
                     .long("fuzzy")
-                    .help("Fuzzy-find results instead of searching for contiguous strings"))
+                    .help("Fuzzy-find results. 0 is off; higher numbers weight shorter/earlier matches more. Try 2"))
+                .arg(Arg::with_name("delete_without_confirm")
+                    .long("delete_without_confirm")
+                    .help("Delete entry without confirm"))
                 .arg(Arg::with_name("output_selection")
                     .short("o")
                     .long("output-selection")
@@ -235,6 +255,25 @@ impl Settings {
         settings.limit = env::var("MCFLY_HISTORY_LIMIT")
             .ok()
             .and_then(|o| o.parse::<i64>().ok());
+
+        settings.interface_view = match env::var("MCFLY_INTERFACE_VIEW") {
+            Ok(val) => match val.as_str() {
+                "TOP" => InterfaceView::Top,
+                "BOTTOM" => InterfaceView::Bottom,
+                _ => InterfaceView::Top,
+            },
+            _ => InterfaceView::Top,
+        };
+
+        settings.result_sort = match env::var("MCFLY_RESULTS_SORT") {
+            Ok(val) => match val.as_str() {
+                "RANK" => ResultSort::Rank,
+                "LAST_RUN" => ResultSort::LastRun,
+                _ => ResultSort::Rank,
+            },
+            _ => ResultSort::Rank,
+        };
+
         settings.session_id = matches
             .value_of("session_id")
             .map(|s| s.to_string())
@@ -274,6 +313,9 @@ impl Settings {
             Some("zsh") => HistoryFormat::Zsh {
                 extended_history: false,
             },
+            Some("zsh-extended") => HistoryFormat::Zsh {
+                extended_history: true,
+            },
             Some("fish") => HistoryFormat::Fish,
             Some(format) => panic!("McFly error: unknown history format '{}'", format),
         };
@@ -294,12 +336,6 @@ impl Settings {
                 );
 
                 settings.append_to_histfile = add_matches.is_present("append_to_histfile");
-                if add_matches.is_present("zsh_extended_history") {
-                    match settings.history_format {
-                        HistoryFormat::Zsh { .. } => settings.history_format = HistoryFormat::Zsh { extended_history: true },
-                        HistoryFormat::Bash | HistoryFormat::Fish => panic!("McFly error: cannot specify zsh extended history with non-zsh history format"),
-                    }
-                }
 
                 if add_matches.value_of("exit").is_some() {
                     settings.exit_code =
@@ -330,7 +366,7 @@ impl Settings {
                         &settings.mcfly_history,
                         settings.history_format,
                     )
-                    .unwrap_or_else(String::new);
+                    .unwrap_or_default();
                 }
 
                 // CD shows PWD as the resulting directory, but we want it from the source directory.
@@ -364,9 +400,22 @@ impl Settings {
                     settings.results = results;
                 }
 
-                settings.fuzzy =
-                    search_matches.is_present("fuzzy") || env::var("MCFLY_FUZZY").is_ok();
+                if let Ok(fuzzy) = env::var("MCFLY_FUZZY") {
+                    if let Ok(fuzzy) = i16::from_str(&fuzzy) {
+                        settings.fuzzy = fuzzy;
+                    } else if fuzzy.to_lowercase() != "false" {
+                        settings.fuzzy = 2;
+                    }
+                }
+                if let Ok(fuzzy) = value_t!(search_matches.value_of("fuzzy"), i16) {
+                    settings.fuzzy = fuzzy;
+                } else if search_matches.is_present("fuzzy") {
+                    settings.fuzzy = 2;
+                }
 
+                settings.delete_without_confirm = search_matches
+                    .is_present("delete_without_confirm")
+                    || env::var("MCFLY_DELETE_WITHOUT_CONFIRM").is_ok();
                 settings.output_selection = search_matches
                     .value_of("output_selection")
                     .map(|s| s.to_owned());
@@ -378,7 +427,7 @@ impl Settings {
                         &settings.mcfly_history,
                         settings.history_format,
                     )
-                    .unwrap_or_else(String::new)
+                    .unwrap_or_default()
                     .trim_start_matches("#mcfly: ")
                     .trim_start_matches("#mcfly:")
                     .to_string();
@@ -433,6 +482,12 @@ impl Settings {
             Some(_val) => true,
             None => false,
         };
+
+        settings.disable_menu = match env::var_os("MCFLY_DISABLE_MENU") {
+            Some(_val) => true,
+            None => false,
+        };
+
         settings.key_scheme = match env::var("MCFLY_KEY_SCHEME").as_ref().map(String::as_ref) {
             Ok("vim") => KeyScheme::Vim,
             _ => KeyScheme::Emacs,
@@ -441,17 +496,38 @@ impl Settings {
         settings
     }
 
+    // Use ~/.mcfly only if it already exists, otherwise create 'mcfly' folder in XDG_CACHE_DIR
     pub fn mcfly_training_cache_path() -> PathBuf {
-        Settings::storage_dir_path().join(PathBuf::from("training-cache.v1.csv"))
-    }
+        let cache_dir = Settings::mcfly_xdg_dir().cache_dir().to_path_buf();
 
-    pub fn storage_dir_path() -> PathBuf {
-        home_dir()
-            .unwrap_or_else(|| panic!("McFly error: Unable to access home directory"))
-            .join(PathBuf::from(".mcfly"))
+        Settings::mcfly_base_path(cache_dir).join(PathBuf::from("training-cache.v1.csv"))
     }
 
+    // Use ~/.mcfly only if it already exists, otherwise create 'mcfly' folder in XDG_DATA_DIR
     pub fn mcfly_db_path() -> PathBuf {
-        Settings::storage_dir_path().join(PathBuf::from("history.db"))
+        let data_dir = Settings::mcfly_xdg_dir().data_dir().to_path_buf();
+
+        Settings::mcfly_base_path(data_dir).join(PathBuf::from("history.db"))
+    }
+
+    fn mcfly_xdg_dir() -> ProjectDirs {
+        ProjectDirs::from("", "", "McFly").unwrap()
+    }
+
+    fn mcfly_base_path(base_dir: PathBuf) -> PathBuf {
+        Settings::mcfly_dir_in_home().unwrap_or(base_dir)
+    }
+
+    fn mcfly_dir_in_home() -> Option<PathBuf> {
+        let user_dirs_file = UserDirs::new()
+            .unwrap()
+            .home_dir()
+            .join(PathBuf::from(".mcfly"));
+
+        if user_dirs_file.exists() {
+            Some(user_dirs_file)
+        } else {
+            None
+        }
     }
 }
